<?php
class Poll {
  // (A) CONSTRUCTOR - CONNECT TO DATABASE
  private $pdo = null;
  private $stmt = null;
  public $error = "";
  function __construct(){
    $this->pdo = new PDO(
      "mysql:host=".DB_HOST.";dbname=".DB_NAME.";charset=".DB_CHARSET,
      DB_USER, DB_PASSWORD, [
      PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
      PDO::ATTR_DEFAULT_FETCH_MODE => PDO::FETCH_ASSOC
    ]);
  }

  // (B) DESTRUCTOR - CLOSE DATABASE CONNECTION
  function __destruct(){
    if ($this->stmt!==null) { $this->stmt = null; }
    if ($this->pdo!==null) { $this->pdo = null; }
  }

  // (C) EXECUTE SQL QUERY
  function query ($sql, $data=null) {
    $this->stmt = $this->pdo->prepare($sql);
    $this->stmt->execute($data);
  }

  // (D) GET POLL & VOTES
  function get ($pid) {
    // (D1) GET POLL QUESTION
    $this->query("SELECT `poll_question` FROM `poll_questions` WHERE `poll_id`=?", [$pid]);
    $question = $this->stmt->fetchColumn();
    if ($question == false) { return false; }
    
    // (D2) GET POLL OPTIONS & VOTES
    $this->query("SELECT * FROM `poll_options` WHERE `poll_id`=?", [$pid]);
    $options = [];
    while ($row = $this->stmt->fetch()) {
      $options[$row["option_id"]] = [
        "t" => $row["option_text"],
        "v" => $row["option_votes"]
      ];
    }

    // (D3) RESULT
    return ["q"=>$question, "o"=>$options];
  }

  // (E) SAVE VOTE
  function save ($pid, $up=null, $down=null) {
    // (E1) GET CURRENT VOTES
    $votes = $this->get($pid);
    if ($votes==false) {
      $this->error = "Invalid poll";
      return false;
    }

    // (E2) UPDATE VOTES
    $sql = "UPDATE `poll_options` SET `option_votes`=? WHERE `poll_id`=? AND `option_id`=?";
    if (is_numeric($up)) {
      $this->query($sql, [$votes["o"][$up]["v"]+1, $pid, $up]);
    }
    if (is_numeric($down)) {
      $this->query($sql, [$votes["o"][$down]["v"]-1, $pid, $down]);
    }
    return true;
  }
}

// (F) DATABASE SETTINGS - CHANGE TO YOUR OWN!
define("DB_HOST", "localhost");
define("DB_NAME", "test");
define("DB_CHARSET", "utf8");
define("DB_USER", "root");
define("DB_PASSWORD", "");

// (G) START SESSION & NEW POLL OBJECT
session_start();
$POLL = new Poll();